@@ -1,4 +1,6 @@
-# Copyright 2011 OpenStack Foundation
+# vim: tabstop=4 shiftwidth=4 softtabstop=4
+
+# Copyright 2011 OpenStack LLC.
 # All Rights Reserved.
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -17,10 +19,17 @@
 Prunes the Image Cache
 """
 
-from glance.image_cache import base
+import logging
+
+from glance.image_cache import ImageCache
+
+logger = logging.getLogger(__name__)
 
 
-class Pruner(base.CacheApp):
+class Pruner(object):
+    def __init__(self, conf, **local_conf):
+        self.conf = conf
+        self.cache = ImageCache(conf)
 
     def run(self):
         self.cache.prune()
