@@ -17,67 +17,31 @@
 
 """Stubouts, mocks and fixtures for the test suite"""
 
-import datetime
-import httplib
-import operator
 import os
 import shutil
-import StringIO
-import sys
 
-import stubout
+try:
+    import sendfile
+    SENDFILE_SUPPORTED = True
+except ImportError:
+    SENDFILE_SUPPORTED = False
+
 import webob
 
+from glance.api.middleware import version_negotiation
+from glance.api.v1 import router
 import glance.common.client
 from glance.common import context
 from glance.common import exception
-from glance.registry import server as rserver
-from glance.api import v1 as server
-from glance.api.middleware import version_negotiation
-import glance.store
-import glance.store.filesystem
-import glance.store.http
-import glance.registry.db.api
+from glance.registry.api import v1 as rserver
+from glance.tests import utils
 
 
-FAKE_FILESYSTEM_ROOTDIR = os.path.join('/tmp', 'glance-tests')
 VERBOSE = False
 DEBUG = False
 
 
-def clean_out_fake_filesystem_backend():
-    """
-    Removes any leftover directories used in fake filesystem
-    backend
-    """
-    if os.path.exists(FAKE_FILESYSTEM_ROOTDIR):
-        shutil.rmtree(FAKE_FILESYSTEM_ROOTDIR, ignore_errors=True)
-
-
-def stub_out_filesystem_backend():
-    """
-    Stubs out the Filesystem Glance service to return fake
-    pped image data from files.
-
-    We establish a few fake images in a directory under //tmp/glance-tests
-    and ensure that this directory contains the following files:
-
-        //tmp/glance-tests/2 <-- file containing "chunk00000remainder"
-
-    The stubbed service yields the data in the above files.
-    """
-
-    # Establish a clean faked filesystem with dummy images
-    if os.path.exists(FAKE_FILESYSTEM_ROOTDIR):
-        shutil.rmtree(FAKE_FILESYSTEM_ROOTDIR, ignore_errors=True)
-    os.mkdir(FAKE_FILESYSTEM_ROOTDIR)
-
-    f = open(os.path.join(FAKE_FILESYSTEM_ROOTDIR, '2'), "wb")
-    f.write("chunk00000remainder")
-    f.close()
-
-
-def stub_out_registry_and_store_server(stubs):
+def stub_out_registry_and_store_server(stubs, base_dir):
     """
     Mocks calls to 127.0.0.1 on 9191 and 9292 for testing so
     that a real Glance server does not need to be up and
@@ -95,7 +59,7 @@ def stub_out_registry_and_store_server(stubs):
         def close(self):
             return True
 
-        def request(self, method, url, body=None, headers=None):
+        def request(self, method, url, body=None, headers={}):
             self.req = webob.Request.blank("/" + url.lstrip("/"))
             self.req.method = method
             if headers:
@@ -107,9 +71,13 @@ def stub_out_registry_and_store_server(stubs):
             sql_connection = os.environ.get('GLANCE_SQL_CONNECTION',
                                             "sqlite://")
             context_class = 'glance.registry.context.RequestContext'
-            options = {'sql_connection': sql_connection, 'verbose': VERBOSE,
-                       'debug': DEBUG, 'context_class': context_class}
-            api = context.ContextMiddleware(rserver.API(options), options)
+            conf = utils.TestConfigOpts({
+                    'sql_connection': sql_connection,
+                    'verbose': VERBOSE,
+                    'debug': DEBUG
+                    })
+            api = context.ContextMiddleware(rserver.API(conf),
+                                            conf, context_class=context_class)
             res = self.req.get_response(api)
 
             # httplib.Response has a read() method...fake it out
@@ -119,9 +87,29 @@ def stub_out_registry_and_store_server(stubs):
             setattr(res, 'read', fake_reader)
             return res
 
+    class FakeSocket(object):
+
+        def __init__(self, *args, **kwargs):
+            pass
+
+        def fileno(self):
+            return 42
+
+    class FakeSendFile(object):
+
+        def __init__(self, req):
+            self.req = req
+
+        def sendfile(self, o, i, offset, nbytes):
+            os.lseek(i, offset, os.SEEK_SET)
+            prev_len = len(self.req.body)
+            self.req.body += os.read(i, nbytes)
+            return len(self.req.body) - prev_len
+
     class FakeGlanceConnection(object):
 
         def __init__(self, *args, **kwargs):
+            self.sock = FakeSocket()
             pass
 
         def connect(self):
@@ -132,16 +120,16 @@ def stub_out_registry_and_store_server(stubs):
 
         def putrequest(self, method, url):
             self.req = webob.Request.blank("/" + url.lstrip("/"))
+            if SENDFILE_SUPPORTED:
+                fake_sendfile = FakeSendFile(self.req)
+                stubs.Set(sendfile, 'sendfile', fake_sendfile.sendfile)
             self.req.method = method
 
         def putheader(self, key, value):
             self.req.headers[key] = value
 
         def endheaders(self):
-            hl = [i.lower() for i in self.req.headers.keys()]
-            assert not ('content-length' in hl and
-                        'transfer-encoding' in hl), \
-                'Content-Length and Transfer-Encoding are mutually exclusive'
+            pass
 
         def send(self, data):
             # send() is called during chunked-transfer encoding, and
@@ -149,7 +137,7 @@ def stub_out_registry_and_store_server(stubs):
             # only write the actual data in tests.
             self.req.body += data.split("\r\n")[1]
 
-        def request(self, method, url, body=None, headers=None):
+        def request(self, method, url, body=None, headers={}):
             self.req = webob.Request.blank("/" + url.lstrip("/"))
             self.req.method = method
             if headers:
@@ -158,17 +146,20 @@ def stub_out_registry_and_store_server(stubs):
                 self.req.body = body
 
         def getresponse(self):
-            options = {'verbose': VERBOSE,
-                       'debug': DEBUG,
-                       'bind_host': '0.0.0.0',
-                       'bind_port': '9999999',
-                       'registry_host': '0.0.0.0',
-                       'registry_port': '9191',
-                       'default_store': 'file',
-                       'filesystem_store_datadir': FAKE_FILESYSTEM_ROOTDIR}
+            conf = utils.TestConfigOpts({
+                    'verbose': VERBOSE,
+                    'debug': DEBUG,
+                    'bind_host': '0.0.0.0',
+                    'bind_port': '9999999',
+                    'registry_host': '0.0.0.0',
+                    'registry_port': '9191',
+                    'default_store': 'file',
+                    'filesystem_store_datadir': base_dir,
+                    'policy_file': os.path.join(base_dir, 'policy.json'),
+                    })
             api = version_negotiation.VersionNegotiationFilter(
-                context.ContextMiddleware(server.API(options), options),
-                options)
+                context.ContextMiddleware(router.API(conf), conf),
+                conf)
             res = self.req.get_response(api)
 
             # httplib.Response has a read() method...fake it out
@@ -192,6 +183,71 @@ def stub_out_registry_and_store_server(stubs):
               client.host == '0.0.0.0'):
             return FakeRegistryConnection
 
+    def fake_image_iter(self):
+        for i in self.source.app_iter:
+            yield i
+
+    stubs.Set(glance.common.client.BaseClient, 'get_connection_type',
+              fake_get_connection_type)
+    stubs.Set(glance.common.client.ImageBodyIterator, '__iter__',
+              fake_image_iter)
+
+
+def stub_out_registry_server(stubs, **kwargs):
+    """
+    Mocks calls to 127.0.0.1 on 9191 for testing so
+    that a real Glance Registry server does not need to be up and
+    running
+    """
+
+    class FakeRegistryConnection(object):
+
+        def __init__(self, *args, **kwargs):
+            pass
+
+        def connect(self):
+            return True
+
+        def close(self):
+            return True
+
+        def request(self, method, url, body=None, headers={}):
+            self.req = webob.Request.blank("/" + url.lstrip("/"))
+            self.req.method = method
+            if headers:
+                self.req.headers = headers
+            if body:
+                self.req.body = body
+
+        def getresponse(self):
+            sql_connection = kwargs.get('sql_connection', "sqlite:///")
+            context_class = 'glance.registry.context.RequestContext'
+            conf = utils.TestConfigOpts({
+                    'sql_connection': sql_connection,
+                    'verbose': VERBOSE,
+                    'debug': DEBUG
+                    })
+            api = context.ContextMiddleware(rserver.API(conf),
+                                            conf, context_class=context_class)
+            res = self.req.get_response(api)
+
+            # httplib.Response has a read() method...fake it out
+            def fake_reader():
+                return res.body
+
+            setattr(res, 'read', fake_reader)
+            return res
+
+    def fake_get_connection_type(client):
+        """
+        Returns the proper connection type
+        """
+        DEFAULT_REGISTRY_PORT = 9191
+
+        if (client.port == DEFAULT_REGISTRY_PORT and
+            client.host == '0.0.0.0'):
+            return FakeRegistryConnection
+
     def fake_image_iter(self):
         for i in self.response.app_iter:
             yield i
